package automorphology.core;

import java.util.*;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

 /**
 * ICorpus interface describes the functionality expected from a corpus in
 * the auto-morphology application. The corpus need to answer questions
 * regarding certain statistics on words, stems, suffixes and even lettes. The
 * functionality has to be full so the heuristics and the MDL algorithm could
 * have every data they need from the corpus as fast as they can and in the most
 * convinient way.
 * @author Yaniv Alamaru, Ofer Senderovich
 * <p>Company: Haifa University</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @version 1.0
 */
public interface ICorpus {

      /**
       * The method returns an iterator which enumerates the words in the corpus.
       * @return an iterator to the beginning of the word container of the corpus.
       */
      public Iterator  getWordIterator();
      public Set getWords();

      /**
       * The method returns the number fo words in the corpus.
       * @return The corpus word count.
       */
      public int length();

      public int getWordFrequency(String word);

      /**
       * The method retrieves the number of appearances of a stem in the corpus.
       * @param stem The input stem interface.
       * @return The stem's frequency.
       */
      public int getStemFrequency(IStem stem);

      /**
       * The method retrieves the number of appearances of a suffix in the corpus.
       * @param suffix The input suffix interface.
       * @return The suffix's frequency.
       */
      public int getSuffixFrequency(ISuffix suffix);

      /**
       * The mehtod retrieves the number of appearances of a suffix from a total
       * appearances of words in the corpus that matches the input signature.
       * @param signature The input signature interface.
       * @param suffix The input suffix interface.
       * @return The suffix frequency in the signature's frequency.
       */
      public int getSuffixInSignatureFrequency(ISignature signature, ISuffix suffix);

      /**
       * The method retrieves the number of appearances of a signature in the corpus.
       * An appearance of the signature is defined as follows: if a word in the corpus
       * is defined by the signature than the word is an appearnace of the signature.
       * @param signature The input signature interface.
       * @return The siganture's frequency.
       */
      public int getSignatureFrequency(ISignature signature);

      /**
       * The method retrieves the number of appearances of a letter. Every occurenace
       * of the letter inside a word is counted for. i.e 'letter' has two appearances of 't'
       * two appearances of 'e' and only one of 'l'.
       * @param letter The input letter character.
       * @return The letter's frequency.
       */
      public int getLetterFrequency(char letter);

      /**
       * The method retrieves the total number of k-grams in the corpus. A k-gram
       * is k-1 letters in the end of a word. The method is necessary for a bootstrap
       * heuristic which uses k-gram analysis.
       * @param k The input k integer.
       * @return The total number of k-grams
       */
      public int getTotalKgram(int k);

      /**
       * The method retrieves the number of successors letter a  stem has in the
       * corpus. The method is used in the bootstrap heuristic of cutting a word
       * in the peak of successors.
       * @param stem The input stem interface.
       * @return The number of successors letters of a stem.
       */
      public int getNumberOfSuccessors(IStem stem);
      public int getSuffixFrequency(ISuffix suffix,IMorphology morphology);
      public int getStemFrequency(IStem stem,IMorphology morphology);
      public int getStemInSignatureFrequency(ISignature signature, IStem stem);
}