package automorphology.core;

import java.util.*;
import java.io.*;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

/**
 * The IMorphology interface represents a morphological analysis of
 * a corpus. The morphological analysis is in the form of the morphemes
 * that the morphology defined (stems and suffixes) and the signatures.
 * The interface defines the functionality which is needed in order to
 * calculate the description length of an entire morphology of a corpus.
 * The morphology is also used in the algorithm in which calculates the
 * corpus compressed length.
 * @see IMDL.
 * @author Yaniv Alamaru, Ofer Senderovich
 * <p>Company: Haifa University</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @version 1.0
 */
public interface IMorphology {

      public void addSignature(ISignature signature);

      /**
       * The method calculates the description length of the entire
       * morphology. The description length is composed from the
       * description length of all the stems, suffixes and signatures
       * in the morphology.
       * @param corpus The corpus which the morphology is derived from.
       * @return The length of the entire morphology in bits.
       */
      public double descriptionLength(ICorpus corpus);

      /**
       * The method returns the signature which the input word belongs
       * to. The method is needed for the algorithm which calculates the
       * length of the compressed corpus.
       * @param word The word which is searched for its signature.
       * @return The word's signature or null if the word does not appear
       * in the morphology.
       */
      public ISignature getSignature(String word);

      /**
       * The method retrieves the words which are analyzed by the
       * morphology as a set. A morphology does not necessary contains
       * a cut for every word into a stem and a suffix. Those words which
       * are not cut are considered as not analyzed.
       * @return The morphology set of analyzed words.

      public Set getAnalayzedWords();
      */
      /**
       * The method sets the number of instances of the analyzed words
       * in the corpus. The purpose of this method is to calculate the
       * number once and to cache it inside the morphology.
       * @param Wa

      public void setNumberOfAnalayzedWord(Integer Wa);
      */
      /**
       * The method calculates the total number of instances of analyzed
       * words inside the corpus. This detail is important in order to
       * calculate the length of a suffix pointer in the morphology.
       * @param corpus The corpus which the morphology belongs to.
       * @return The total number of instances of analyzed words in the corpus.
       */
      public int getNumberOfAnalyzedWord(ICorpus corpus);

      /**
       * The method calculates the total number of the simple words
       * inside the corpus. a simple word is a word which hasn't got
       * a recursive analysis. The morphology defines which words are
       * analyzed as recrusive or simple.
       * @param corpus The corpus which the morphology is based on.
       * @return The number of simple words in the corpus.
       */
      public int getNumberOfSimpleWords(ICorpus corpus);
      public void print();
      public JSet getSignatures();
      public void create(Map stemToSuffixs);
      public void optimize();
      public ISignature getNullSigature();
      public int getNumberOfWords();
      public void setStemFrequency(IStem stem,int num);
      public int getStemFrequency(IStem stem);
      public void setSuffixFrequency(ISuffix suffix,int num);
      public int getSuffixFrequency(ISuffix suffix);
      public IMorphology getNewMorphology();
      public void printAnalysis(PrintStream out,boolean prefix);
}