package automorphology.core;

import java.util.Set;
/**
 *
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

/**
 * The ISginature interface represent a signature in a morphology. A
 * signature is a set of two lists: a stem list and a suffix list. When
 * the two lists are cartesian muliplied then we get a set of words.
 * The signature is a part of a morphology. The signature is a result
 * of cutting words into stems and suffixes. All the stems which have
 * the same suffixes in a corpus are collected together into a signature.
 * The interface defines the functionality which is needed in order to
 * calculate the description length of a signature in a certain morphology
 * of a corpus.
 * @author Yaniv Alamaru, Ofer Senderovich
 * <p>Company: Haifa University</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @version 1.0
 */
public interface ISignature extends Comparable{

      /**
       * The method returns the description length of the signature. The
       * description length of a signature is defined as follows:
       * The sum of all the signature's stems pointer lengths +
       * The sum of all the signature's suffixes pointer lengths
       * ( the suffix freuqnecy here is measured only from the total
       * occurences of the signature's words in the corpus and not from
       * all the words in the corpus) + the signature's pointer length
       * (which is depends on the signature frequency in the corpus).
       * @param corpus The corpus to use in order to find the signature's frequency.
       * @param morphology The morphology which the signature derives from.
       * @return The signature's description length in bits.
       */
      public double descriptionLength(ICorpus corpus,IMorphology morphology);

      /**
       * The method returns the length of the signature's pounter. The length
       * of the pointer depends on the signature's frequency in the corpus.
       * @param corpus The corpus to use in order to find the signature's frequency.
       * @return The signature's pointer length in bits.
       */
      public double pointerLength(ICorpus corpus);

      /**
       * The method retrieves the signature's words as a set. The words
       * are the result of a cartesian multiplication of the stem list and
       * the suffix list.
       * @return The signature's set of words.
       */
      public JSet getWords();

      /**
       * The method adds a new stem to the signature stems list.
       * @param stem The new stem to be added
       */
      public void addStem(IStem stem);

      public JSet getSuffixs();
      public JSet getStems();
      public boolean equals(ISignature signature);
      public IStem getStem(String word);
      public ISuffix getSuffix(String word);
      public void print();
      public void borrowLetter(char letter);
      public ISignature copy();
      public void removeStem(IStem stem);
      public boolean containsStem(IStem stem);
      public int compareTo(Object o);
}