package automorphology.core;

import java.util.*;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

public class Signature implements ISignature{

      private JSet stems = new JSet();
      private JSet suffixs;
      private JSet words = null;
      private boolean dirty = false;

      public Signature(JSet suffixs) {
            this.suffixs = suffixs;
      }


      public double descriptionLength(ICorpus corpus,IMorphology morphology) {
            /* Stems Component */
            double stemComponent = Math.log(stems.size())/Math.log(2);
            Iterator stemIter = stems.iterator();
            while(stemIter.hasNext()) {
                  IStem stem = (IStem)stemIter.next();
                  stemComponent += stem.pointerLength(corpus,morphology);
            }
            /* Suffixs Component */
            double suffixComponent = Math.log(suffixs.size())/Math.log(2);
            Iterator suffixIter = suffixs.iterator();
            while(suffixIter.hasNext()) {
                  ISuffix suffix = (ISuffix)suffixIter.next();
                  if (corpus.getSuffixInSignatureFrequency(this,suffix) == 0)
                        System.out.println(suffix.toString());
                  double tmp = corpus.getSignatureFrequency(this)/(double)corpus.getSuffixInSignatureFrequency(this,suffix);
                  suffixComponent += Math.log(tmp)/Math.log(2);
            }
            return stemComponent + suffixComponent + this.pointerLength(corpus);
      }

      public double pointerLength(ICorpus corpus) {
            double tmp = corpus.length()/(double)corpus.getSignatureFrequency(this);
            return Math.log(tmp)/Math.log(2);
      }

      public JSet getWords() {
            if ((dirty != true) && (words != null))
               return words;
            words = new JSet();
            IStem stem = null;
            Iterator stem_iter = stems.iterator();
            while(stem_iter.hasNext()) {
                  stem = (IStem)stem_iter.next();
                  ISuffix suffix = null;
                  Iterator suffix_iter = suffixs.iterator();
                  while(suffix_iter.hasNext()) {
                        suffix = (ISuffix)suffix_iter.next();
                        StringBuffer word = new StringBuffer(stem.toString());
                        if (!(suffix.toString().equals("#")))
                              word.append(suffix.toString());
                        words.add(word.toString());
                  }
            }
            this.words = words;
            dirty = false;
            return words;
      }

      public void addStem(IStem stem) {
            dirty = true;
            stems.add(stem);
      }

      public void removeStem(IStem stem) {
            dirty = true;
            Iterator stemIter = stems.iterator();
            while(stemIter.hasNext()) {
                  IStem aStem = (IStem)stemIter.next();
                  if (aStem.toString().compareTo(stem.toString()) == 0) {
                        stemIter.remove();
                        return;
                  }
            }
      }

      public JSet getSuffixs() {
            dirty = true;
            return suffixs;
      }

      public JSet getStems() {
            dirty = true;
            return stems;
      }

      public boolean equals(ISignature signature) {
            return suffixs.equals(signature.getSuffixs());
      }

      public int compareTo(Object obj) {
            ISignature signature = (ISignature)obj;
            if (!(stems.iterator().hasNext())) {
               return -1;
            }
            IStem stem1 = (IStem)stems.iterator().next();
            IStem stem2 = (IStem)signature.getStems().iterator().next();
            return stem1.toString().compareTo(stem2.toString());
      }

      public IStem getStem(String word) {
            Iterator stemIter = stems.iterator();
            while (stemIter.hasNext()) {
                  IStem stem = (IStem)stemIter.next();
                  if (word.startsWith(stem.toString())) {
                        Iterator suffixIter = suffixs.iterator();
                        while(suffixIter.hasNext()) {
                              ISuffix suffix = (ISuffix)suffixIter.next();
                              StringBuffer mergedWord = new StringBuffer(stem.toString());
                              if (!(suffix.toString().equals("#")))
                                    mergedWord.append(suffix.toString());
                              if (mergedWord.toString().equals(word))
                                    return stem;
                        }
                  }
            }
            return null;
      }

      public ISuffix getSuffix(String word) {
            if (this.getStem(word).equals(new SimpleStem(word)))
                        return new Suffix("#");
            Iterator suffixIter = suffixs.iterator();
            while (suffixIter.hasNext()) {
                  ISuffix suffix = (ISuffix)suffixIter.next();
                  if (word.endsWith(suffix.toString())) {
                        Iterator stemIter = stems.iterator();
                        while(stemIter.hasNext()) {
                              IStem stem = (IStem)stemIter.next();
                              StringBuffer mergedWord = new StringBuffer(stem.toString());
                              mergedWord.append(suffix.toString());
                              if (mergedWord.toString().equals(word))
                                    return suffix;
                        }
                  }
            }
            return null;
      }

      public void borrowLetter(char letter) {
            JSet newStems = new JSet();
            JSet newSuffixs = new JSet();
            Iterator stemIter = this.stems.iterator();
            while(stemIter.hasNext()) {
                  IStem stem = (IStem)stemIter.next();
                  stem = new SimpleStem(stem.toString()+letter);
                  newStems.add(stem);
            }
            Iterator suffixIter = this.suffixs.iterator();
            while(suffixIter.hasNext()) {
                  ISuffix suffix = (ISuffix)suffixIter.next();
                  if (suffix.toString().length() == 1)
                        suffix = new Suffix("#");
                  else
                        suffix = new Suffix(suffix.toString().substring(1));
                  newSuffixs.add(suffix);
            }
            stems = newStems;
            suffixs = newSuffixs;
      }

      public void print() {
            Iterator stemIter = stems.iterator();
            Iterator suffixIter = suffixs.iterator();
            while(stemIter.hasNext()) {
                  IStem stem = (IStem)stemIter.next();
                  System.out.print(stem.toString()+",");
            }
            System.out.println();
            while(suffixIter.hasNext()) {
                  ISuffix suffix = (ISuffix)suffixIter.next();
                  System.out.print(suffix.toString()+",");
            }
            System.out.println();
            System.out.println("--------------------------------------");
      }

      public ISignature copy() {
            JSet suffixSet = new JSet();
            Iterator suffixIter = this.suffixs.iterator();
            while (suffixIter.hasNext()) {
                  ISuffix suffix = new Suffix(((ISuffix)suffixIter.next()).toString());
                  suffixSet.add(suffix);
            }
            ISignature signature = new Signature(suffixSet);
            Iterator stemIter = stems.iterator();
            while (stemIter.hasNext()) {
                  IStem stem = new SimpleStem(((IStem)stemIter.next()).toString());
                  signature.addStem(stem);
            }
            return signature;
      }

      public boolean containsStem(IStem stem) {
            Iterator stemIter = stems.iterator();
            while(stemIter.hasNext()) {
                  IStem stem2 = (IStem)stemIter.next();
                  if (stem2.equals(stem))
                        return true;
            }
            return false;
      }
}